/********************************************************************
 *
 *  Alerts for Steuerung 13.06.2013
 *
 *  R. Kohleick
 *
 *  licensed under GNU GPLv3
 *
 ********************************************************************/


#ifndef alerters
#define alerters

#include "Arduino.h"
#include <Menus.h>
#include <mystuff.h>


#define HIGH_ALARM	1
#define LOW_ALARM	2
#define LOGIC_ALARM	3

#define MSGLEN  8

enum alertstate { disarmed, armed, triggered, acknowledged };

class Alert {
	
public:	
	Alert(int val=0, int t=HIGH_ALARM, char *m=0, char *msg=0);	// create alert

	void disarm(void);							// disarm alert, no further action taken	
	
	void arm(void);								// arm alert and set threshold to new temperature
	
	boolean check(int temp);					// if alarm is armed, and threshold passed, returns true until acknowledged
	boolean check(boolean flag);				// if alarm is armed, and flag set, returns true until acknowledged

	void acknowledge(void);							// if acknowledges trigger event
	
	boolean isarmed(void) { return (state==armed); };
	boolean istriggered(void) { return (state==triggered); };
	boolean isacknowledged(void) { return (state==acknowledged); };
	boolean haschanged(void) { return (state==acknowledged || state==triggered); };

	int level(void) { return value; }
	void setvalue(int v) { disarm(); value=v;	}
	
	void status(void);							// prints alert status to outside world
	
private:
	byte type;

	int value;
	char *alertstr;
	char *text;
	alertstate state;
};

#endif