/********************************************************************
 *
 *  Controler - controls heater and does switching, timing etc.
 *  for Steuerung 13.06.2013
 *
 *  R. Kohleick
 *
 *  licensed under GNU GPLv3
 *
 ********************************************************************/



#ifndef Controler_h
#define Controler_h


#define NEVER       0
#define SUNDAYS     1
#define MONDAYS     2
#define TUESDAYS    3
#define WEDNESDAYS  4
#define THURSDAYS   5
#define FRIDAYS     6
#define SATURDAYS   7
#define WORKDAYS    8    // Mo - Fr
#define WEEKENDS    9    // Sa - Su
#define FIRSTHALF  10    // Mo - We
#define SECONDHALF 11    // Th - Fr
#define ALWAYS     12    // everyday

#define MAXPROFILE    4    // maximum number of temperature profiles in program

#define MAXSETPOINTS  5    // maximum number of setpoints per profile


const unsigned long maxwaitfortempincrease=2700;      // maximum wait time in sec for temperature to increase, if no significant increase, then hardware malfunction ?

class Schedule {
	
	friend class Control;
	
public:
	struct event {
		int temp;    // temperature in 1/10 centigrade representation
		int time;    // time in minutes of day representation
	};
	
	byte active;
	byte number;
	
	Schedule(int t=130);
	
	void add(int temp, byte hrs, byte min);
	void del(byte n);
	
	boolean editschedule(void);
	int     findsoll(int  t);
	boolean isactivetoday(byte dow);
	struct event setpoints[MAXSETPOINTS];
	
private:
	void sort(void);			// sorts the schedule by ascending time
	void swap( int *a, int *b);	// swaps two integer values
	void menufromschedule(Menu *m);
	unsigned int lastentrytoedit;
};


#define    CTRL_OFF      '-'
#define    CTRL_MANUAL   'M'
#define    CTRL_CMFT     'C'
#define    CTRL_ECON     'E'
#define    CTRL_CYCLE    'O'
#define    CTRL_AFREEZE  'A'
#define    CTRL_SCHEDULE 'P'

#define mod(h, m) (60*(h)+(m))
#define mod2min(m) ((m)%60)
#define mod2hrs(m) ((m)/60)

class Control {
	
	friend class CLI;	

    
public:
	Control(void);
    
	void Controlreset(void);

	void begin(int heater=13);
	int mode;            // indicates, which mode heater is running in
	
	boolean heaterstatus;   // indictes whether heater is switched on or off

	int manualtemp;      // manually entered temp
	int comfort;         // comfort temp
	int economy;         // economy temp
	int antifreeze;      // antifreeze settings
	
	// for temp cycle, night economy, day comfort	
	int comfort_on;      // time when comfort should be switched on
	int comfort_off;     // time when comfort should be switched off
	
	int regulate(); // effectively switches heater on and off
	
    Schedule  profile[MAXPROFILE];		// provide storage for tempprofiles
	
    void disableallschedules(void);		// disable all schedules in profile
    void deleteallschedules(void);		// delete all schedules in profile
    boolean isanyscheduleactive(void);		// as the name says
	
	void status(char *s, byte length);       // prints status to string s with no more chars than length
	
	int heaterminruntime;		//  minimum runtime of the heater in seconds
    int hysteresis;			// hysteresis of regulator,  1 deg C
	
	int logtime;			// time when logging should occur24*2
	boolean logging;		// signals that we should do a logging now.
	boolean malfunction;	// we suspect that heater is not working properly
	
	void dolog(void)	{ logtime=abs(logtime); };
	void dontlog(void)	{ logtime=-abs(logtime); };
    
	void update(int in, int out, byte sns);
    void initminmax(int in, int out);
    
    int in_tempmax;
    int in_tempmin;
    int out_tempmax;
    int out_tempmin;
	
	
private:
	int sensors;			// number of sensors
	int in_temp;			// inside temperature
	int out_temp;			// outside temperature
    
	unsigned long laston;	// last time that heater was started
	int starttemp;			// temperature at which heater was started;
	
	byte heaterpin;
	
	boolean checkformalfunction(int tmax);

	int temp_soll(int t, byte dow);
	
	void checkfortimers(int h, int m);
	void update_runtime(unsigned int t);
};

#endif
