#include <ctype.h>
#include <avr/pgmspace.h>
#include <myLiquidCrystal.h>
#include <Menus.h>
#include <EEPROM.h>
#include <mystuff.h>
#include <streeng.h>
#include <Phonebook.h>

    //extern myLiquidCrystal lcd;
extern myLiquidCrystal lcd;

int const eepromoffset=300;

#define EEPROMINDEX	36

#define telnochar(c)	(isdigit(c) || ((c)=='+'))


// Offset,	Offset + 3, 5 telnos each telnolength

byte phonebook::offset(byte i ) {
    return (position+3+i*telnolength);
}

byte phonebook::numberofentries(void) {
	return EEPROM.read(position+2);
}

byte phonebook::setnumberofentries(byte n) {
	EEPROM.write(position+2, n);
    return n;
}

void phonebook::phonebookreset(void) {
	setnumberofentries(0);
}

void phonebook::readtelno(char *s, byte i) {
	unsigned int pos;
	byte j;
	
    *s=0;

	if(i<maxentry) {
        pos=offset(i);
            
        for(j=0;j<telnolength-1; j++)
            *s++=EEPROM.read(pos+j);
        *s=0;
	}
}


void phonebook::writetelno(char *s, byte i) {
	unsigned int pos;
	byte j;
	
	if(i<maxentry) {
        pos=offset(i);
        
        for(j=0;j<telnolength; j++)
            EEPROM.write(pos+j,*s++);
	}
}



phonebook::phonebook(unsigned int pos, unsigned int magic) {
	unsigned int i;

	position=pos;
	
	i=EEPROM.read(pos+0)<<8 | EEPROM.read(pos+1);
	
	if (i!=magic) { // magicnumber not found, initialize the book        
		EEPROM.write(pos+0, (magic >> 8) & 0xFF);		// write magic hex number
		EEPROM.write(pos+1, magic & 0xFF);
		setnumberofentries(0);
	}
}


byte phonebook::addentry(char *newentry) {
	char *p, *s;
	byte n;
    
    n=numberofentries();
	if (n==maxentry)
		return 0;		// book full

    p=newentry;
	truncleft(p);
    
	for(s=p;telnochar(*s); s++);
	if(*s)                              // not 0 character, i.e. end of string
		return numberofentries();		// telno not in right format, return

	writetelno(newentry, n);
	
	return setnumberofentries(n+1);
}




byte phonebook::deleteentry(byte m) {
	unsigned int j, pos1, pos2, end;
	byte n;
    
    n=numberofentries();
    
	if (m>=n)
		return 0;

	pos1=offset(m);
	pos2=offset(m+1);
	end=offset(n);
	
	for(j=0; j<end-pos2; j++)
		EEPROM.write(pos1+j, EEPROM.read(pos2+j));

	return setnumberofentries(n-1);
}



phonebook *_pbk;


void _cpypbdynamic(byte i, char *s) {
	_pbk->readtelno(s,i);       // for each number, the meny entry is equal to this number
}



void phonebook::menufromphonebook(Menu *m) {
	byte i;
	unsigned int pos;
	char t[MENUSTRLEN];	
	
	m->menureset();
	
	m->entry(0x1000, 0);        // Main Menu
	m->entry(0x2000, 1);        // Add Entry

	lastentrytoedit=0;

	for(i=0; i<numberofentries(); i++) {        // then for each number in memory
		
		pos=0x1000+(i+2)*0x1000;
		
		m->entry(pos, 0x80+i);                  // Make entry with that number, set flag to make this entry dynamic
		m->entry(pos+0x0100, 2);                // Edit

		lastentrytoedit=pos+0x0100; 

		m->entry(pos+0x0200, 3);                // Delete
	}
	
    m->obtainstring(t,4);
	m->setpanel(t);
	m->home();
}



void phonebook::addblanksright(char *s) {
	byte i;
	
	for(i=0;*s;s++) // go to end
        i++;
    
	for(i=i;i<telnolength;i++){
		*s++=' ';		
	}
	*(--s)=0;   // go one back and mark end of line
}



boolean phonebook::isinphonebook(char *s) {
	char t[telnolength+1];
	boolean found = false;
	byte i;
	
	for(i=0; (i<numberofentries()) && !found; i++) {
		readtelno(t,i);
		found=(strcmp(s,t)==0);
	}
	return found;
}


void phonebook::editphonebook(myLiquidCrystal *lcd) {
	
	int i, pos;
	int newpos;
	unsigned int command;
	boolean quit=false;
	char s[telnolength+1];
	int skipnext=false;
	
    const byte menusize=1+2+3*maxentry;
    
	menuitem pbm[menusize];
	
	streeng eeprom(eepromoffset);
	eeprom.setbaseindex(EEPROMINDEX);
	
	// create a menu instance
	Menu pbmenu(pbm, menusize, &eeprom, 0, _cpypbdynamic);
	
	// initialize buttons, sequence: up, enter, down
	pbmenu.begin(11, 10, 9);
	
	do {
		_pbk=this;
		menufromphonebook(&pbmenu);
		
		if(!skipnext)
			command=pbmenu.event();
		else {
			command=lastentrytoedit;
			skipnext=false;
		}
        
		switch(command){
			case 0x1000:
				quit=true;
				break;
				
			case 0x2000:
				newpos=addentry("+0");
				skipnext=(newpos!=0);           // successful newentry
				break;

			default:
				pos=(command & 0xF000)>>12;     // pos the number's position in the phoneboook, 
                pos-=3;                         // need to subtract 3 as number we have 2 menu items (main / add) and numbers start with 0
				i=(command & 0x0F00)>>8;        // i denotes the submenu (edit/delete)
                
				switch (i) {
					case 1:
						pbmenu.cleararound(2);
						pbmenu.forceredraw();
						
						readtelno(s, pos);
						addblanksright(s);
						pbmenu.editnumstr(s, 0, 2, '+');
						truncright(s);
						writetelno(s, pos);
						break;
						
					case 2:
						deleteentry(pos);
						break;
						
					default:
						break;
				}
				break;				
		}
	} while (!quit);
	
	return;
}
