/***************************
 *
 *
 *	Library to access GSM Modems
 *
 *  tested for
 *
 *  Siemens tc35i       19200 baud
 *  Siemens mc35i       19200 baud
 *  Sagem   HiLo       115200 baud
 *
 *
 *	R. Kohleick	29.12.2011
 *
 * licensed under GNU GPLv3
 *
 **/


#ifndef gsmmodem
#define gsmmodem

#define SERIAL	0	// Defines Serial port: 0 for 2009 & UNO, 0..3 for MEGA

#include <ctype.h>
#include "Arduino.h"

const int smalldelay=75;
const int mediumdelay=150;
const int largedelay=500;
const int postcommanddelay=5*smalldelay;

const long timeout=1000;

const int maxargs=15;

class GSMModule {
	
public:
	GSMModule(char *buffer=0, byte size=0, char *telno=0 );
	
	byte begin(void);
	byte startmodule(void);
	int incoming(void);
    
	boolean available;      // indicates that we found a modem on the serial link
	
	int sendcommand(char *c);
	byte parseline(char *c);
	
	void networkoperator(void);
	boolean networkregistered(void);
    
	int readsms(byte ndx);
	byte readfirstsms(void);		

	byte sendsms(char *telno, char *text);

	boolean callwaiting(void);
	byte smswaiting(void);

	void deleteallsms(void);
	void deletesms(byte ndx);
    
	char *argv[maxargs];
	byte argc;

private:
	byte maxsmsstorage;
	
	char *buffer;
	char *telno;
	byte buffsize;
	
	int readln(char *b, byte size);
	
	void waitforchar(char c);
	
	void deletehyphens(char *s);
};

#endif
