/***************************
 *
 *
 *	collection of things that i find useful
 *
 *
 *	R. Kohleick	28.10.2010
 *
 * licensed under GNU GPLv3
 *
 **/

#include <avr/pgmspace.h>
#include <EEPROM.h>
#include <Arduino.h>
#include <ctype.h>

#define chartoint(c)	((c)-'0')
#define EOS			(char)0

#define STRLEN		17


/**********************************************************************
 *
 * copies dayof week to string
 *
 * day to be in range 1..7
 *
 * 1 = Sunday
 *
 *********/

char *daystrings[]={"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"};

void sprintday(char *s, byte d) {
	d--;
	    
    strcpy(s,daystrings[d]);
}


/**********************************************************************
 *
 * copies dayof month of year to string
 *
 * month to be in range 1..12
 *
 *********/

char strmon1[] PROGMEM  = "January";
char strmon2[] PROGMEM  = "February";
char strmon3[] PROGMEM  = "March";
char strmon4[] PROGMEM  = "April";
char strmon5[] PROGMEM  = "May";
char strmon6[] PROGMEM  = "June";
char strmon7[] PROGMEM  = "July";
char strmon8[] PROGMEM  = "August";
char strmon9[] PROGMEM  = "September";
char strmon10[] PROGMEM  = "October";
char strmon11[] PROGMEM  = "November";
char strmon12[] PROGMEM  = "December";

PGM_P monstrings[] PROGMEM = {  
	strmon1,
	strmon2,
	strmon3,
	strmon4,
	strmon5,
	strmon6,
	strmon7,
	strmon8,
	strmon9,
	strmon10,
	strmon11,
	strmon12 };

void sprintmon(char *s, byte d) {
	
	d=max(1,d);
	d=min(12,d);
	
	strcpy_P(s, (PGM_P)pgm_read_word(&(monstrings[d-1])));
}


/**********************************************************************
 *
 * prints time to string
 *
 *
 **********/

void sprinttime(char *s, byte h, byte m) {
	
	s[0]=h/10+'0';
	s[1]=h%10+'0';
	s[2]=':';
	s[3]=m/10+'0';
	s[4]=m%10+'0';
	s[5]=EOS;
}

/**********************************************************************
 *
 * changes leading blanks to leading zeros in string s
 *	 s the 0 terminated string
 *
 **********/ 

void leadingzeros(char *s) {
	
	while(*s) {
		if(*s==' ')
			*s='0';
		s++;
	}

}


void sprintint(int number, char *t, byte field, byte dec) {
	
	byte i=0;
	boolean sign=false;
	char s[STRLEN];
	char *p;
	
	if((field>STRLEN-1) || (dec>=field) || (field==0)) {
		*t=EOS;
		return;
	}
	
	if(number<0) {
		sign=true;
		number=-number;
	}

	p=s;
	do {
		*p++='0'+number%10;
		number/=10;
		i++;

		if(i==dec) {
			*p++='.';
			i++;
		}
		
	} while (((number>0)||(i<dec+2)) && (i<STRLEN));
    
	if(sign && (i<STRLEN)) {
		*p++='-';
		i++;
	}// i has now strlength
	
	for(;i<field;i++)
		*t++=' ';

	for(p--; p>=s; p--)
		*t++=*p;
		
	*t=EOS;
}


/**********************************************************************
 *
 * converts string into integer value, skipping non digit chars
 *
 * s - the string to be converted
 *
 ********/

unsigned int scanint(char *s) {
	unsigned int result=0;

	while(!isdigit(*s) && (*s)) // seek first numeric
		s++;
	
	while(*s) {
		if(isdigit(*s)) {
			result*=10;
			result+=chartoint(*s);
		}
		s++;
	}
	
	return result;
}

/**********************************************************************
 *
 * prints temperature as string
 *
 * s - the string, temp the temperature 
 *
 ********/

void sprinttemp(char *s, int temp) {
    sprintint(temp, s, 5, 1);
	strcat(s,"C");
}

/*************
 *
 * writes an entire memoryblock to eeprom
 *
 */

void savetoeeprom(unsigned int offset, unsigned int magic, byte *p, unsigned int size) {              // save the memblock to eeprom
	unsigned int i;
	
	EEPROM.write(offset+0, (magic >> 8) & 0xFF);		// write magic hex number
	EEPROM.write(offset+1, magic & 0xFF);
	
	for(i=0; i<size; i++)
		EEPROM.write(offset+2+i, *p++);
}

/*************
 *
 * reads an entire memoryblock from eeprom
 *
 */

boolean readfromeeprom(unsigned int offset, unsigned int magic, byte *p, unsigned int size) {              // reads the memblock from eeprom	
	unsigned int i;
	byte m1, m2;
	
	m1=EEPROM.read(offset+0);
	m2=EEPROM.read(offset+1);
	
	i=m1<<8 | m2;
	
	if(i!=magic) {
		return false;
	}
	
	for(i=0; i<size; i++)
		*p++=EEPROM.read(i+offset+2);
	
	return true;
}

/*****************************
 *
 * reads line terminated by CR into buffer
 * eliminates CR / LF pairs
 *
 * returns length of read string
 *
 *****/


int Readln(char *b) {	
	int i=0;
	byte c=0;
	
  	do {
		if(Serial.available()) {
			*b++=c=Serial.read();
			i++;
		}
	} while ((c!=';') && (c!='\r') && (c!='\n') && (i<140));
	
	*b=(char)0;
	return i;
}


/*****************************
 *
 * shifts string nchars to the right
 *
 */

void shiftstring(char *s, byte n) {
  
  int len, i;
  
  len=strlen(s);
  
  for(i=len; i>=0; i--)
    s[i+n]=s[i];
  
  for(i=0; i<n; i++)
    s[i]='*';
}


/*****************************
 *
 * eliminates character c from string
 *
 */

void eliminate(char *s, char c) {
	char *p;
	
	p=s;
	
	for(p=s; *s; s++) {
		if(*s!=c)
			*p++=*s;
	}
	*p=0;
}

/*****************************
 *
 * truncates the string by eliminating non chars
 * including space at the end
 *
 *****/


void truncright(char *b) {
	char *s;
	
	for(s=b;*s;s++); // go to end of string
	
	for(s--; ((s>=b) && (isspace(*s))); s--)
		*s=0;
}

/*****************************
 *
 * truncates the string by eliminating non chars
 * including space at the beginning
 *
 *****/


void truncleft(char *b) {
	char *s;
	
	s=b;
	for(; isspace(*b); b++);
	
	for(;(*b!=0);b++) {
		*s++=*b;
	}
	*s=0;
}

/*****************************
 *
 * loops for mil milliseconds
 *
 *****/


void mydelayint(unsigned int mil) {

    unsigned long now, then;

    now=millis();
    then=now+mil;

    if(then<now)    // rollover
        while(then<millis());   // loop for rollover to occur

    while(millis()<then);   // thereafter wait for time to expire
}

/*****************************
 *
 * loops for mil milliseconds
 *
 *****/


void mydelaylong(unsigned long mil) {

    unsigned long now, then;
    
    now=millis();
    then=now+mil;
   
    if(then<now)    // rollover
        while(then<millis());   // loop for rollover to occur
    
    while(millis()<then);   // thereafter wait for time to expire
}
        
/*****************************
 *
 * concats s1, s2 but only up to len chars
 * returns length of resulting string
 *
 *****/


byte mystrcat(char *s1, char *s2, byte len) {
    byte l=0;   
    char *s, *t;
    
    for(s=s1; *s; s++)
        l++;
    
    for(t=s2;*t && l<len-1;t++) {
        *s++=*t;
        l++;
    }
    
    *s=0;
    return l;
}