/********************************************************************
 *
 *  V5.7 13.06.2013
 *
 *  R. Kohleick
 *
 *  licensed under GNU GPLv3
 *
 ********************************************************************
 *
 * The circuit:
 *
 *  LCD 20x4 chars or 16x4 chars
 *
 *   LCD RS to digital pin 8.
 *   LCD EN to digital pin 7
 *   LCD D4 to digital pin 6
 *   LCD D5 to digital pin 5
 *   LCD D6 to digital pin 4
 *   LCD D7 to digital pin 3
 *
 * 10K trimmer:
 *   ends to +5V and ground
 *   wiper to LCD VO (pin 3 LCD)
 *
 * Button DOWN	switching to GND, digital pin 9
 * Button ENTER	switching to GND, digital pin 10
 * Button UP	switching to GND, digital pin 11
 *
 * One Wire Interface  digital pin 2
 *    connected to temperature sensor DS18B20 (x2)
 *    connected to real time clock    DS2415/DS2417
 * 
 * Heater Control      digital pin 13 drives relay
 *
 * LCD Backlight       digital pin 12 drives transistor
 *
 *******************************/

//#define DISPLAY16x4 
#define DISPLAY20x4 

#include <avr/pgmspace.h>
#include <mystuff.h>
#include <EEPROM.h>
#include <streeng.h>
#include <myLiquidCrystal.h>
#include <RTC_DS2415.h>
#include <Phonebook.h>
#include <Alerts.h>
#include <OneWire.h>
#include <Controler.h>
#include <CommandLine.h>
#include <Menus.h>
#include <gsmmodem.h>
#include <MsTimer2.h>
#include "Steuerung.h"

streeng eeprom(eepromoffset);

char buffer[buffsize];
char tempstr[tempbuffsize];
char telno[telnolength];

OneWire  ow(2);  // one wire bus for DS18B20 temp sensor

GSMModule gsm(buffer,buffsize,telno);  // gsm module

const int heaterpin=13;  //  pins for heater

commsource communicationrequest;

myLiquidCrystal lcd(8, 7, 6, 5, 4, 3);  // create a Liquid Crystal display

// and create a menu instance

const int mainmenusize=46;

menuitem mainmenu[mainmenusize];

Menu menu(mainmenu, mainmenusize, &eeprom, service);

Clock rtclock(&ow);  // create a SoftwareClock
byte rtcaddress[8];  // and its address

Control controler;  // create a Controller

CLI interpreter;    // and associated interpreter

phonebook receivers(EEPROM_RECEIVERS, EEPROM_MAGIC_RECEIVERS);  // create the appropriate phonebook

Alert alertlow(80, LOW_ALARM, buffer, "Tin <");    // create alerts for noification through SMS
Alert alerthigh(300, HIGH_ALARM, buffer, "Tin >");
Alert malfunction(0, LOGIC_ALARM, buffer, "Malfunction ?");

// variables for DS18B20 one wire temperature measurement

byte onewiresensors=0;  // number of sensors

sensor tempsensors[2];  // available tempsensors

sensor *outside;  // ptrs to hold in- and outside temperatures
sensor *inside;
int insidetemp;
int outsidetemp;
int setpoint;    // setpoint returned from controler



/****************************
*
* heartbeat routine
*
****/

byte systime;

void heartbeat(void) {
  systime++;
  
  if( systime%5 == 0) {     // regulate heater every 5 seconds
    controler.update(insidetemp, outsidetemp, onewiresensors);  // feed controler with temperature and sensor data
    setpoint=controler.regulate();
  }
}



/****************************
*
* delay for n sec, n<256
*
****/

void mydelay(byte sec) {
  unsigned long ende;
  
  ende=millis()+1000*sec; 
  while(millis() < ende);
}


/****************************************************
 *
 * Finds required one wire devices
 *
 *****/

int lookuponewires(void) {
  byte i;
  byte address[8];

  byte cnt=0;

  while (ow.search(address)) {
    switch (address[0]) {

    case 0x27:       //0x27 = DS2415 / DS2417 time chip
      for(i=0;i<8;i++)
        rtcaddress[i]=address[i];                                    
      break;

    case 0x28:     // DS 18B20
      if(cnt<2) {
        for(i=0;i<8;i++)
          tempsensors[cnt].address[i]=address[i];
        cnt++;
      }
      break;

    default:
      break;
    }
  }
  return cnt;
}



/****************************************************
 *
 * compares two sensor addresses
 * 
 *  returns:
 *    true if equal
 *    false if unequal
 * 
 *****/

int sensor_compare(byte *a1, byte *a2) {
  byte i;

  for(i=0;i<8;i++) {
    if (*a1++!=*a2++)
      return false;
  }
  return true;
}


/****************************************************
 *
 * writes inside and outside temperature sensor addresses to eeprom
 * 
 *
 *****/


void writeonewirestoeeprom(void) {
  byte i;
  byte address[16];

  for(i=0;i<8;i++) {
    address[i]=inside->address[i];
    address[i+8]=outside->address[i];
  }
  savetoeeprom(EEPROM_ONEWIRES, EEPROM_MAGIC_ONEWIRES, address, ONEWIRE_SIZE);	
}

/****************************************************
 *
 * checks whether onewires are known, and allocates position (inside outside)
 * 
 *
 *  returns:
 *          true if onewires were previously allocated and were identified
 *          false if both onewires were not identified (if one was identified, the other is automatically assumed to be the other).
 *          does not return: if no onewires
 *
 *****/

#define INSIDE    0
#define OUTSIDE   8

int appointonewires(void) {
  byte i;
  byte in, out;
  byte address[16];
  char buffer[20];

  if(onewiresensors==0) {    // we need at least one sensor, if nothing is there, hangup
    myprint("No Sensors",1,1);
    while(true);
  }

  if(!readfromeeprom(EEPROM_ONEWIRES, EEPROM_MAGIC_ONEWIRES, address, 16)) {
    inside=&tempsensors[0];
    outside=&tempsensors[1];
    writeonewirestoeeprom();      // but assume this one is inside and remember current setup
    myprint("1st run?",1,1);
    if(onewiresensors>1) {
      myprint("switch ?",2,1);
    }
    return false;    // no valid data found in EEPORM, we have to ask for manual input
  }

  // at this point of time we know that there were previous sensors

  if(onewiresensors==1) {    // there is only one sensor available, check, whether is is a former insidesonsor
    inside=&tempsensors[0];
    outside=&tempsensors[1];
    for(i=0;i<8;i++)              // delete upper
        outside->address[i]=0;

    writeonewirestoeeprom();      // assume this one is right and remember current setup
    if(sensor_compare(inside->address,address)) { // is this the former inside ?              
      myprint("in sensor found",1,1);
      return(true);                // take this sensor, presumably outside was just cut off
    }
    return false;                        // if unknown sensor, better check manually
  }

  // at this point we have two onewire sensors & previous sensor data

  if(sensor_compare(tempsensors[0].address, &address[INSIDE])) {
    inside=&tempsensors[0];
    outside=&tempsensors[1];
    writeonewirestoeeprom();	   // found old insidesensor, outside is other one
    return true;
  }

  if(sensor_compare(tempsensors[1].address, &address[INSIDE])) {
    inside=&tempsensors[1];
    outside=&tempsensors[0];
    writeonewirestoeeprom();	   // found old insidesensor, outside is other one
    return true;
  }

  // two sensors found, but old insidesensor was not found, we have to ask for manual input

  inside=&tempsensors[0];
  outside=&tempsensors[1];
  writeonewirestoeeprom();  

  myprint("switch ?",1,1);

  return false;    // and let user check manually
}

/*************
 *
 * read the temperature with 1/10 deg centigrade accuracy from DS18B20 sensor
 *
 * in order to avoid timing issues with the DS, the routine will only
 * read temperature from the sensor every 2nd call. 
 *
 * the calling routine must assure that between 1st and 2nd call, there
 * is alwyays sufficient time for the DS to read the temp (approx. 1 sec).
 *
 */

int DSReadTemp10(struct sensor *thesensor){

  byte data[9];
  byte i;
  int temp;

  ow.reset();		            //reset the bus
  ow.select(thesensor->address);    //select our sensor

  if(!thesensor->polled)
    ow.write(0x44);		    //CONVERT T function call (44h) which puts the temperature into the scratchpad
    
  else {		
    ow.write(0xBE);      	    //read the scratchpad (BEh)
    for (i=0;i<9;i++)
      data[i] = ow.read();
    temp= (data[1]<<8) | data[0];   // put temperature into temp, this is a signed value
    temp*=10;                       // we want to store 1/10 deg centigrade values
    temp>>=4;                       // temp now holds the temperature, >>4 conserves sign bit
    
    if (thesensor->temp==startupsign)
      thesensor->temp=temp;
      
    if( abs(thesensor->temp-temp) < abs(thesensor->temp>>3) ) {    // if delta to previous measurement is small enough (>>3 = 1/8 = 12.5%)
      thesensor->temp=temp;         // take this temperature
      thesensor->attempts=0;        // reset counter
    } else {                                     // if delta to previous measurement too large
      if(thesensor->attempts++>maxattempts) {  // check whether we have more than maxattempts
        thesensor->temp=temp;                  // then take this measurement, so we only eliminate short term faults
        thesensor->attempts=0;                 // and reset the counter
      }
    }
  }
  thesensor->polled=!thesensor->polled;
  return thesensor->temp;  // return temperature * 10, so 1/10 deg centigrade resolution
}


/*****************************
 *
 *  reads temperature from inside and outside and updates data
 *
 *  sequence:
 *    poll insidesensor, let main routine do something else
 *    if there, poll outsidesensor, let main routine do something else
 *    query insidesensor tempdata, let main routine do something else
 *    if there, query outsidesensor tempdata, let main routine do something else
 *
 ***/

void updatetemp(void) {

static byte cnt=0;

  switch(cnt++) {
    case 0:
    insidetemp=DSReadTemp10(inside);
    break;

    case 1:
    if(onewiresensors>1)                 // and if available also outside
      outsidetemp=DSReadTemp10(outside);
    cnt=0;
    break;    
  }
  
}


/*****************************
 *
 *  deals with communication requests from gsm modem
 *
 ***/

unsigned int handlegsmrequests(int *arguments) {
  unsigned int command=NOCOMMAND;
  int ndx;

  switch(ndx=gsm.incoming()) {
  case 0:                // nothing
    break;

  case -1:    // call waiting
    gsm.callwaiting();  // due to command being overriden, a call overrides an sms command, read telno of calling number

    if (receivers.isinphonebook(telno)) {                               // call from known number triggers status response
      if (alertlow.haschanged())          // and as a goody, (re)activate alarms if they were triggered
        alertlow.arm();
      if (alerthigh.haschanged())        // no action if not activated or triggered
        alerthigh.arm();
      if (malfunction.haschanged())
        malfunction.arm();

      controler.initminmax(insidetemp,outsidetemp);
      gsm.deleteallsms();
      communicationrequest=sms;   // send status via sms
      command=0x8000;             // status command
    }
    break;

  default:  // positive number received, sms waiting at index ndx
    gsm.readsms(ndx);
    if (receivers.isinphonebook(telno)) {    // the sms was sent from someone we know, so interpret the command and pass it on to 
      communicationrequest=sms;
      command=interpreter.interpret(buffer,arguments);
    }
    gsm.deletesms(ndx);          // delete sms received in the module, if valid response, we have now the command for it
    break;
  }
  return command;
}


/******************************
 *
 * 1 sec service routine
 *
 * basically changes the menu panel with new time and temp data
 *
 ***/

void updatedisplay(void) {
  static boolean flash=false;
  char s[25];    // add some extra space for outside temp printing
  byte j;
  
  rtclock.sprinttime(s);

#ifdef DISPLAY20x4
  if(onewiresensors>1) {// output room temperature and rounded outside temperature
                        //  DDDDDDDDiiiiiioooo S
                        //  WW XX:XX+XX.X+XX.X S
                        //  01234567890123456789

    sprinttemp(&s[8],insidetemp);
    sprinttemp(&s[13],outsidetemp);
  } else {              //  output room temperature only
                        //  DDDDDDDD  iiiiii   S
                        //  WW XX:XX  +XX.XC   *
                        //  01234567890123456789

    s[8]=s[9]=' ';
    sprinttemp(&s[10],insidetemp);
    s[16]=s[17]=' ';
  }
  j=18;
#endif

#ifdef DISPLAY16x4
  sprinttemp(&s[8],insidetemp);// for small display only output room temp
                        //  DDDDDDDDiiiiii S 
                        //  WW XX:XX +XX.X *
                        //  0123456789012345
  j=14;
#endif

  s[j++]=' ';
  if(flash)
    s[j++]=controler.heaterstatus?'*':' ';
  else
    s[j++]=controler.mode;

  s[j]=0;

  flash=!flash;
  menu.setpanel(s);
  if (!menu.displaylocked)
    menu.display_panel();
}



/*****************************
 *
 * service routine, called repeatedly and frequently by the menu system
 * handles all tasks 
 *
 * select the intervals such that they do not overlap too often, they should be
 * relative prime
 *
 **/

unsigned int service(int *arguments) {

  static byte oldtime;
  unsigned int command;

  if(gsm.available) {
    command=handlegsmrequests(arguments); // first, check whether we need to do something with the gsm modem
    if(command!=NOCOMMAND)
      return command;                 // if so, issue the command
    
    if(systime==180)                 // every 3 min or so
      gsm.sendcommand("AT");         // keep Modem alive
    }
    
  if(systime==oldtime)                // if not, check whether second has passed since last call
    return NOCOMMAND;

  // and do the following approx every 1 sec       

  oldtime=systime;                    // 1 sec has passed, remember current time

  rtclock.updateclock();              // update time
  updatetemp();                       // update temperature readings  
  updatedisplay();                    // then update display

  if(communicationrequest!=none)           // there is still something in the buffer from previous function call, wait for this to be finished
    return NOCOMMAND;

  if(controler.logging) {                  // meanwhile controler has asked to issue a status log (via sms)
    controler.logging=false;
    communicationrequest=firstinlist;      // send it to superuser
    return LOG_EVENT;
  }
  
  // now check, whether we have any alarms pending, if so send sms to all

  if(alertlow.check(insidetemp) || alerthigh.check(insidetemp) || malfunction.check(controler.malfunction))  {
    communicationrequest=smsbroadcast;
    return ALERT_EVENT;
  }

  if(malfunction.istriggered() && !controler.heaterstatus)  // if the malfunction alarm is triggered, and heater has meanwhile (been) switched off
    malfunction.arm();                                      // then reactivate the alarm to monitor the next heating phase

  return NOCOMMAND;                               // otherwise, nothing to do, return
}

/******************************
 *
 * toggle those ON/OFF mneu items if needed
 *
 ***/

#define MENUSTR_ON   6
#define MENUSTR_OFF  20


/*******************
 *
 *  toggles one mneu entry depending on its state on/off
 *
 ***/

void menutoggle(unsigned int entry, boolean status) {
  if (status)
    menu.changetxt(entry,MENUSTR_OFF);
  else
    menu.changetxt(entry,MENUSTR_ON);
}

/*******************
 *
 *  toggles all menu entries depending on their state on/off
 *
 ***/

void updatemenutoggles(void) {
  menutoggle(0x2100, controler.mode==CTRL_CYCLE);
  menutoggle(0x4110, alertlow.isarmed());
  menutoggle(0x4210, alerthigh.isarmed());
}

/******
 *
 *  myprint - prints answer string to lcd
 *
 */

void myprint(char *s, byte line,  byte indent) {
  lcd.setCursor(0,line);
  if(indent)
    lcd.write('>');
  lcd.print(s);
}


/******
 *
 *  setup the devices and menu structures
 *
 */

void setup() {
  byte i;
  
  Serial.begin(19200);

  //// ################ should be 
  ////  Serial.println(EEPROM_STARTOFSTRINGS,DEC);  // check the eprom space required for controler / available for strings

#ifdef DISPLAY20x4
  lcd.begin(20, 4); 	// set up the LCD's number of rows and columns: 
#endif

#ifdef DISPLAY16x4
  lcd.begin(16, 4); 	// set up the LCD's number of rows and columns: 
#endif

  menu.begin(11, 10, 9, 12);   // now, initialize buttons, sequence: up, enter, down, backlight

  eeprom.setbaseindex(eepromindex);

  menu.entry(0x1000, 0);
  menu.entry(0x1100, 1);
  menu.entry(0x1200, 2);
  menu.entry(0x1300, 3);
  menu.entry(0x1400, 4);
  menu.entry(0x1500, 38);
  menu.entry(0x1510, 47);
  menu.entry(0x1520, 48);

  menu.entry(0x2000, 5);
  menu.entry(0x2100, MENUSTR_ON);    // On
  menu.entry(0x2200, 7);
  menu.entry(0x2210, 8);
  menu.entry(0x2220, 9);

  menu.entry(0x3000, 10);
  menu.entry(0x3100, 11);	
  menu.entry(0x3200, 12);	
  menu.entry(0x3300, 13);
  menu.entry(0x3400, 14);
  menu.entry(0x3500, 15);

  menu.entry(0x4000, 16);  // Alerts
  menu.entry(0x4100, 17);  // Lower
  menu.entry(0x4110, MENUSTR_ON);  // first value: On or Off
  menu.entry(0x4120, 18);  // Value
  menu.entry(0x4200, 19);  // Upper
  menu.entry(0x4210, MENUSTR_ON);  // first value: On or Off
  menu.entry(0x4220, 18);  // Value
  
  menu.entry(0x4300, 38);  // Status

  menu.entry(0x5000, 21);	

  menu.entry(0x5100, 22);
  menu.entry(0x5110, 23);
  menu.entry(0x5120, 24);
  menu.entry(0x5130, 25);

  menu.entry(0x5140, 26);
  menu.entry(0x5141, 27);
  menu.entry(0x5142, 28);

  menu.entry(0x5200, 29);

  menu.entry(0x5300, 30);
  menu.entry(0x5310, 31);
  menu.entry(0x5320, 32);

  menu.entry(0x5400, 34);
  menu.entry(0x5410, 35);

  menu.entry(0x5500, 36);
  menu.entry(0x5600, 37);

  menu.entry(0x6000, 39);
  menu.entry(0x6100, 49);
  menu.entry(0x6200, 50);

  lcd.clear();
  eeprom.read(buffer,40);
  myprint(buffer,0,0);        
  delay(1000);

  lcd.clear();
  eeprom.read(buffer,42);
  myprint(buffer,0,0);        

  // read controler and alert structures into ram

  readfromeeprom(EEPROM_CONTROL, EEPROM_MAGIC_CONTROL, (byte*)&controler, sizeof(Control));
  controler.begin(heaterpin);

  readfromeeprom(EEPROM_ALERTHIGH, EEPROM_MAGIC_ALERT, (byte*)&alerthigh, sizeof(Alert));
  alerthigh.disarm();
  readfromeeprom(EEPROM_ALERTLOW, EEPROM_MAGIC_ALERT, (byte*)&alertlow, sizeof(Alert));
  alertlow.disarm();

  // register all available one wire sensors i) temperature and ii) real time clock

  onewiresensors=lookuponewires();

  if(!appointonewires()) {
    myprint("Check !",3,1);
    while(menu.getkey()!=ENTERKEY);
  }
  rtclock.begin(rtcaddress);          //    start real time clock

  // start the temperature sampling & show previous message for sufficient long time

  inside->temp=startupsign;
  if(onewiresensors>1)
    outside->temp=startupsign;
  
  for(i=0;i<10;i++) {
    updatetemp();
    delay(1000);
  }
  
  controler.initminmax(insidetemp, outsidetemp);     // now there should be a valid set of temperature data

  //   that being done, initialize communication buffers & search for gsm modem

  lcd.clear();
  myprint("GSM...",0,0);
  eeprom.read(buffer,41);
  myprint(buffer,1,0);

  gsm.begin();

  if (gsm.available) {
    eeprom.read(tempstr,50);
    myprint(tempstr,2,1);
  } 
  else
    myprint("GSM err",2,1);

  myprint(buffer,3,1);    

  // and perform regular interrupt to trigger various events 

  MsTimer2::set(1000, heartbeat);
  MsTimer2::start();

  // arm malfunction alarm to  check whether heating has some effect

  malfunction.arm();
}



void loop() {
  byte hours, minutes, dow, day, month;
  byte i;
  int year, temp;
  unsigned int command;
  
  boolean dontsendstatus=false;    // as a default, do send a status message, if this is set true, the status message is not send
  boolean commanderror=false;      // initially, we assume that there is no error, it is encountered, this flag is set to true
  boolean controlerchanged=false;  // the controler data has been changed
  
  updatemenutoggles();

  *buffer=*telno=0;        // reset all strings to empty

  switch(menu.event()) {
    
  case 0x1100:
    controler.manualtemp=menu.getint(controler.manualtemp, controler.antifreeze, 500, 4, 1, "Temp ? ", 1, 2);
  case 0x8007:		// temperature on
    controler.mode=CTRL_MANUAL;
    controlerchanged=true;
    menu.home();
    break;

  case 0x1200:
  case 0x8005:		// comfort on
    controler.mode=CTRL_CMFT;
    controlerchanged=true;
    menu.home();
    break;

  case 0x1300:
  case 0x8003:		// economy on
    controler.mode=CTRL_ECON;
    controlerchanged=true;
    menu.home();
    break;

  case 0x1400:
  case 0x8009:		// antifreeze on
    controler.mode=CTRL_AFREEZE;
    controlerchanged=true;
    menu.home();
    break;


  case 0x1510:          // show temperature status min, max
    lcd.clear();
    myprint("Setpoint",0,0);
    sprinttemp(tempstr,setpoint);
    lcd.setCursor(9,0);
    lcd.print(tempstr);
    myprint("In",1,0);
    sprinttemp(tempstr,controler.in_tempmin);
    lcd.setCursor(3,1);
    lcd.print(tempstr);
    lcd.print(" ");
    sprinttemp(tempstr,controler.in_tempmax);
    lcd.print(tempstr);
    if(onewiresensors>1) {
      myprint("Outside",2,0);
      sprinttemp(tempstr,outsidetemp);
      lcd.setCursor(9,2);
      lcd.print(tempstr);
      myprint("Out",3,0);
      sprinttemp(tempstr,controler.out_tempmin);
      lcd.setCursor(3,3);
      lcd.print(tempstr);
      lcd.print(" ");
      sprinttemp(tempstr,controler.out_tempmax);
      lcd.print(tempstr);
    }
    mydelay(visualization);
    break;

  case 0x1520:            // reset min / max 
    controler.initminmax(insidetemp, outsidetemp);
    controlerchanged=true;
    break;

  case 0x2100:          // toggle cycle mode
    if(controler.mode==CTRL_CYCLE)
      controler.mode=CTRL_MANUAL;
    else
      controler.mode=CTRL_CYCLE;
    controlerchanged=true;
    menu.home();
    break;			

  case 0x8001:		// cycle on by sms
    controler.mode=CTRL_CYCLE; 
    controlerchanged=true;
    menu.home();
    break;			

  case 0x2210:
    do {
      hours=mod2hrs(controler.comfort_on);
      minutes=mod2min(controler.comfort_on);
      menu.gettime(&hours, &minutes, "cmft ? ", 1, 2);
      controler.comfort_on=mod(hours,minutes);
    } while (controler.comfort_on>=controler.comfort_off);
    controlerchanged=true;
    break;

  case 0x2220:
    do {
      hours=mod2hrs(controler.comfort_off);
      minutes=mod2min(controler.comfort_off);
      menu.gettime(&hours, &minutes, "econ ? ", 1, 2);
      controler.comfort_off=mod(hours,minutes);
    } while (controler.comfort_on>=controler.comfort_off);
    controlerchanged=true;
    break;

  case 0x3100:
     controler.profile[0].editschedule();
     if(controler.isanyscheduleactive())
       controler.mode=CTRL_SCHEDULE;
     else
       controler.mode=CTRL_MANUAL;                        
     controlerchanged=true;
     break;     
     
  case 0x3200:
     controler.profile[1].editschedule();
     if(controler.isanyscheduleactive())
       controler.mode=CTRL_SCHEDULE;
     else
       controler.mode=CTRL_MANUAL;                        
     controlerchanged=true;
     break;
     
  case 0x3300:
     controler.profile[2].editschedule();
     if(controler.isanyscheduleactive())
       controler.mode=CTRL_SCHEDULE;
     else
       controler.mode=CTRL_MANUAL;                        
     controlerchanged=true;
     break;
     
  case 0x3400:
     controler.profile[3].editschedule();
     if(controler.isanyscheduleactive())
       controler.mode=CTRL_SCHEDULE;
     else
       controler.mode=CTRL_MANUAL;                        
     controlerchanged=true;
     break;
     
  case 0x3500:
     controler.mode=CTRL_MANUAL;                        
     controlerchanged=true;
     break;

  case 0x4110:
    if(alertlow.isarmed())
      alertlow.disarm();
    else 
      alertlow.arm();
      savetoeeprom(EEPROM_ALERTLOW, EEPROM_MAGIC_ALERT, (byte*)&alertlow, sizeof(Alert));
    break;

  case 0x4120:
    alertlow.setvalue(menu.getint(alertlow.level(), 0, 500, 4, 1, "Temp ? ", 1, 2));
    savetoeeprom(EEPROM_ALERTLOW, EEPROM_MAGIC_ALERT, (byte*)&alertlow, sizeof(Alert));
    break;

  case 0x4210:
    if(alerthigh.isarmed())
      alerthigh.disarm();
    else 
      alerthigh.arm();
      savetoeeprom(EEPROM_ALERTHIGH, EEPROM_MAGIC_ALERT, (byte*)&alerthigh, sizeof(Alert));
    break;

  case 0x4220:
    alerthigh.setvalue(menu.getint(alerthigh.level(), 0, 500, 4, 1, "Temp ? ", 1, 2));
    savetoeeprom(EEPROM_ALERTHIGH, EEPROM_MAGIC_ALERT, (byte*)&alerthigh, sizeof(Alert));
    break;

  case 0x4300:
    lcd.clear();
    alerthigh.status();
    myprint(buffer,1,0);
    alertlow.status();
    myprint(buffer,2,0);
    mydelay(visualization);
    break;

  case 0x5110:
    controler.comfort=menu.getint(controler.comfort, controler.antifreeze, 500, 4, 1, "cmft ? ", 1, 2);
    controlerchanged=true;
    break;

  case 0x5120:
    controler.economy=menu.getint(controler.economy, controler.antifreeze, 500, 4, 1, "econ ? ", 1, 2);
    controlerchanged=true;
    break;

  case 0x5130:
    controler.antifreeze=menu.getint(controler.antifreeze, 0, 500, 4, 1, "A-Freeze ? ", 1, 2);
    controlerchanged=true;
    break;

  case 0x5141:    // Factory settings
    controler.Controlreset();
    receivers.phonebookreset();
    alertlow.setvalue(40);
    alerthigh.setvalue(300);
    controlerchanged=true;
    menu.home();
    break;

  case 0x5142:    // Factory settings abort
    menu.home();
    break;

  case 0x5200:  // recipients
    receivers.editphonebook(&lcd);
    break;

  case 0x5310:
    hours=rtclock.hours();
    minutes=rtclock.minutes();
    if(menu.gettime(&hours, &minutes, "Time ? ", 1, 2))
      rtclock.settime(hours, minutes);
    break;

  case 0x5320:
    rtclock.updateclock();
    day=rtclock.day();
    month=rtclock.mon();
    year=rtclock.year();
    if(menu.getdate(&day, &month, &year, "Date? ", 0, 2)) {
      rtclock.setdate(day, month, year);
    }
    break;

  case 0x5410:
    controler.heaterminruntime=menu.getint(controler.heaterminruntime, 10, 600, 4, 0, "MinHtr ? ", 1, 2);                        
    controlerchanged=true;
    break;

  case 0x5500:
    controler.hysteresis=menu.getint(controler.hysteresis, 0, 50, 4, 1, "Hyst ? ", 1, 2);
    controlerchanged=true;
    break;

  case 0x5600:
    sensor *s;			
    if(onewiresensors>1) {
      s=outside;
      outside=inside;
      inside=s;
      writeonewirestoeeprom();
      controler.initminmax(insidetemp, outsidetemp);
    }
    controlerchanged=true;
    break;

  case 0x6100:
    if(gsm.available)
      communicationrequest=smsbroadcast;
    break;

  case 0x6200:
    if(gsm.available) {
      lcd.clear();
      myprint("Operator  Qlty",1,0);      
      gsm.networkoperator();
      myprint(buffer,2,0);
      gsm.sendcommand("AT+CSQ");
      lcd.setCursor(10,2);
      lcd.print(gsm.argv[1]);
      mydelay(visualization);
    }
    break;

  case 0x8000:		// status, no args
  case LOG_EVENT:
    break;

  case 0x8002:		// cycle set hh:mm temp hh:mm temp
    controler.comfort_on=menu.args[2];
    controler.comfort=menu.args[3];
    controler.comfort_off=menu.args[4];
    controler.economy=menu.args[5];
    controler.mode=CTRL_CYCLE;
    controlerchanged=true;
    break;

  case 0x8004:		// economy set temp
    controler.economy=menu.args[2];
    controler.mode=CTRL_ECON;
    controlerchanged=true;
    break;

  case 0x8006:		// comfort set temp
    controler.comfort=menu.args[2];
    controler.mode=CTRL_CMFT;
    controlerchanged=true;
    break;

  case 0x8008:		// temperature set temp
    controler.manualtemp=menu.args[2];
    controler.mode=CTRL_MANUAL;
    controlerchanged=true;
    break;

  case 0x800A:		// antifreeze set temp
    controler.antifreeze=menu.args[2];
    controler.mode=CTRL_AFREEZE;
    controlerchanged=true;
    break;

  case 0x800E:		// help, no args
    strcpy(buffer,"rtfm!\n");
    dontsendstatus=true;
    break;

  case 0x800F:		// time set hh:mm
    rtclock.settime(menu.args[2]/60, menu.args[2]%60);
    break;

  case 0x8011:          // alert nr on/off
    switch (menu.args[1]) {
    case 0:
      if(menu.args[2]==1)
        alertlow.arm();
      else
        alertlow.disarm();
        savetoeeprom(EEPROM_ALERTLOW, EEPROM_MAGIC_ALERT, (byte*)&alertlow, sizeof(Alert));
      break;

    case 1:
      if(menu.args[2]==1)
        alerthigh.arm();
      else
        alerthigh.disarm();
        savetoeeprom(EEPROM_ALERTHIGH, EEPROM_MAGIC_ALERT, (byte*)&alerthigh, sizeof(Alert));
      break;
    }
    break;

  case 0x8012:          // alert nr set value
    switch (menu.args[1]) {
    case 0:
      alertlow.setvalue(menu.args[3]);
      alertlow.arm();
      savetoeeprom(EEPROM_ALERTLOW, EEPROM_MAGIC_ALERT, (byte*)&alertlow, sizeof(Alert));
      break;

    case 1:
      alerthigh.setvalue(menu.args[3]);
      alerthigh.arm();
      savetoeeprom(EEPROM_ALERTHIGH, EEPROM_MAGIC_ALERT, (byte*)&alerthigh, sizeof(Alert));
      break;
    }                          

  case 0x8014:		// alert status message request
    alertlow.status();
    strcpy(tempstr,buffer);
    alerthigh.status();
    strcat(buffer,"\n");
    strcat(buffer,tempstr);
    dontsendstatus=true;
    break;

  case 0x8013:		// date set dd mm yy
    rtclock.setdate(menu.args[2], menu.args[3], menu.args[4]);
    break;

  case 0x8015:		// status log set time
    if(menu.args[3]!=0)		// never set to 00:00, or else the switch on/off mechanism will not work.
      controler.logtime=menu.args[3];
    controlerchanged=true;
    break;

  case 0x8016:		// status log on / off
    if(menu.args[2])
      controler.dolog();  // switch on
    else
      controler.dontlog();  // make it negative to switch off
    controlerchanged=true;
    break;

  case ALERT_EVENT:        // alert issued, put notification string into buffer, acknowledge
    alertlow.acknowledge();
    alerthigh.acknowledge();
    malfunction.acknowledge();
    dontsendstatus=true;  // but do not send status sms
    break; 

  case NOCOMMAND:          // do nothing
    break;

  default:		  // error catcher
    commanderror=true;    // do not send status message and just the error mesage
    dontsendstatus=true;
    break;
  }


  if(gsm.available) { // in case the modem is connected

  // now that all commands have been executed, we can put a status report into the buffer (if needed)
  // so that the sms sender receives feedback on his command 
  // and terminate the buffer entry by commenting that the device is ready.

    if(!dontsendstatus)  // if previous commands did not override this default
      controler.status(buffer,buffsize);  // send status message as command confirmation sms
  
    if(commanderror)     // followed by indication whether command encountered error or all is ok
      strcat(buffer,"ERROR");
    else
      strcat(buffer,"\nOK");

  // and then actually send the message so generated

    switch(communicationrequest) {
    case call:      // in case one wants to make the modem call some time in the future
      break;

    case voicebroadcast:      // voicecalls to everyone in phonebook (not implemented)
      break;

    case sms:      // just send the sms to originating number
      gsm.sendsms(telno, buffer);
      break;

    case smsbroadcast:  // send the sms to everyone in the phonebook				
      for(i=0;i<receivers.numberofentries();i++) {
        receivers.readtelno(telno,i);
        gsm.sendsms(telno, buffer);
      }
      break;

    case firstinlist:  // send the sms to first phonebook entry				
      if(receivers.numberofentries()>0) {
        receivers.readtelno(telno,0);
        gsm.sendsms(telno, buffer);
      }
      break;

    default:  // all others, including none
      break;
    }
    communicationrequest=none;
  }

  if(controler.mode!=CTRL_SCHEDULE)      // the controler has been set to a non scheduling mode, so we need to deactivate all schedules
    controler.disableallschedules();

  if(controlerchanged)
    savetoeeprom(EEPROM_CONTROL, EEPROM_MAGIC_CONTROL, (byte*)&controler, sizeof(Control));

  menu.forceredraw();
}


